@[Link("onigmo")]
@[Link(ldflags: "#{__DIR__}/onigmo/onigwrap.o")]

lib LibOnigmo
  type Regex = Pointer(Void)
  type Region = Pointer(Void)

  fun create = onigwrap_create(pattern : LibC::Char*, len : UInt32,
                               ignoreCase : Int32,
                               multiline : Int32,
                               dotall : Int32) : Regex
  fun free = onigwrap_free(re : Regex)
  fun region_free = onigwrap_region_free(region : Region)

  fun search = onigwrap_search(re : Regex, str : LibC::Char*, offset : UInt32, length : UInt32) : Region
  fun num_regs = onigwrap_num_regs(region : Region) : Int32
  fun pos = onigwrap_pos(region : Region, index : Int32) : Int32
  fun len = onigwrap_len(region : Region, index : Int32) : Int32
end

module Onigmo
  class Match
    property begin : Int32
    property end : Int32
    property value : String

    def initialize(@begin, @end, @value)
    end
  end

  class Regex
    def initialize(@pattern : String, @ignorecase = false, @multiline = false, @dotall = false)
      @re = LibOnigmo.create(@pattern.to_unsafe, @pattern.bytesize, @ignorecase ? 1 : 0, @multiline ? 1 : 0, @dotall ? 1 : 0)
    end

    def finalize
      LibOnigmo.free(@re)
    end

    def match(str : String, offset = 0)
      region = LibOnigmo.search(@re, str.to_unsafe, offset, str.bytesize)
      result = [] of Match?
      num_regs = LibOnigmo.num_regs(region)
      if num_regs > 0
        (0...num_regs).each do |i|
          b = LibOnigmo.pos(region, i)
          e = b + LibOnigmo.len(region, i)
          if b == -1 || e == -1
            result << nil
          else
            v = str[b...e]
            result << Match.new(b, e, v)
          end
        end
      else
        return [] of Match
      end
      LibOnigmo.region_free(region)
      result
    end
  end
end

pattern = "#.*x"
str = "# foobar"

re = Onigmo::Regex.new(pattern, false, false, false)
p! re.match(str)
